@@ -125,6 +125,22 @@ update_wireframe_window (MetaDisplay         *display,
 #endif
 }
 
+static void
+graphics_sync (BoxAnimationContext *context)
+{
+  XImage *image;
+  
+  /* A hack to force the X server to synchronize with the
+   * graphics hardware
+   */
+  image = XGetImage (context->screen->display->xdisplay,
+		     context->screen->xroot,
+		     0, 0, 1, 1,
+		     AllPlanes, ZPixmap);
+
+  XDestroyImage (image);
+}
+
 static gboolean
 effects_draw_box_animation_timeout (BoxAnimationContext *context)
 {
@@ -182,6 +198,8 @@ effects_draw_box_animation_timeout (BoxAnimationContext *context)
           XDestroyWindow (context->screen->display->xdisplay,
                           context->wireframe_xwindow);
         }
+
+      graphics_sync (context);
       
       g_free (context);
       return FALSE;
@@ -267,7 +285,7 @@ effects_draw_box_animation_timeout (BoxAnimationContext *context)
     }
 
   /* kick changes onto the server */
-  XFlush (context->screen->display->xdisplay);
+  graphics_sync (context);
   
   return TRUE;
 }
@@ -364,7 +382,7 @@ meta_effects_draw_box_animation (MetaScreen     *screen,
                                                   0,
                                                   CopyFromParent,
                                                   CopyFromParent,
-                                                  CopyFromParent,
+                                                  (Visual *)CopyFromParent,
                                                   CWOverrideRedirect | CWBackPixel,
                                                   &attrs);
 
