@@ -2,6 +2,7 @@
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2003 Rob Adams
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -75,7 +76,7 @@ struct _MetaScreen
   MetaWorkspace *active_workspace;
 
   GList *workspaces;
-  
+
   MetaStack *stack;
 
   MetaCursor current_cursor;
@@ -107,7 +108,6 @@ struct _MetaScreen
   
   guint keys_grabbed : 1;
   guint all_keys_grabbed : 1;
-  guint showing_desktop : 1;
   
   int closing;
 
@@ -147,7 +147,7 @@ void          meta_screen_ensure_tab_popup    (MetaScreen                 *scree
 
 void          meta_screen_ensure_workspace_popup (MetaScreen *screen);
 
-void          meta_screen_focus_mouse_window   (MetaScreen                 *screen,
+MetaWindow*   meta_screen_get_mouse_window     (MetaScreen                 *screen,
                                                 MetaWindow                 *not_this_one);
 
 const MetaXineramaScreenInfo* meta_screen_get_current_xinerama    (MetaScreen    *screen);
@@ -196,13 +196,16 @@ void meta_screen_resize (MetaScreen *screen,
                          int         width,
                          int         height);
 
-void     meta_screen_minimize_all_except (MetaScreen *screen,
-                                          MetaWindow *keep);
+void     meta_screen_minimize_all_on_active_workspace_except (MetaScreen *screen,
+                                                              MetaWindow *keep);
 
 /* Show/hide the desktop (temporarily hide all windows) */
 void     meta_screen_show_desktop        (MetaScreen *screen);
 void     meta_screen_unshow_desktop      (MetaScreen *screen);
 
+/* Update whether the destkop is being shown for the current active_workspace */
+void     meta_screen_update_showing_desktop_hint          (MetaScreen *screen);
+
 void     meta_screen_apply_startup_properties (MetaScreen *screen,
                                                MetaWindow *window);
 
